class Organism < ActiveRecord::Base
  include Seek::Rdf::RdfGeneration

  acts_as_favouritable
  grouped_pagination
  acts_as_uniquely_identifiable

  linked_to_bioportal apikey: Seek::Config.bioportal_api_key

  has_many :assay_organisms, inverse_of: :organism
  has_many :models
  has_many :assays, through: :assay_organisms, inverse_of: :organisms
  has_many :strains, dependent: :destroy
  has_many :samples, through: :strains

  has_and_belongs_to_many :projects

  before_validation :convert_ncbi_id

  validates_presence_of :title
  validates :concept_uri, url: { allow_nil: true, allow_blank: true }
  validates :concept_uri, ncbi_concept_uri: true, allow_blank: true

  validate do |organism|
    unless organism.bioportal_concept.nil? || organism.bioportal_concept.valid?
      organism.bioportal_concept.errors.each do |attr, msg|
        errors.add(attr, msg)
      end
    end
  end

  def can_delete?(user = User.current_user)
    !user.nil? && user.is_admin_or_project_administrator? && models.empty? && assays.empty? && projects.empty?
  end

  def can_manage?(user = User.current_user)
    User.admin_logged_in?
  end

  def searchable_terms
    terms = [title]
    if concept
      terms |= concept[:synonyms].collect { |s| s.delete('\""') } if concept[:synonyms]
      terms |= concept[:definitions].collect { |s| s.delete('\""') } if concept[:definitions]
    end
    terms
  end

  def self.can_create?
    User.admin_or_project_administrator_logged_in? || User.activated_programme_administrator_logged_in?
  end

  #overides that from the bioportal gem, to always make sure it is based on http://purl.bioontology.org/ontology/NCBITAXON/
  def ncbi_uri
    return nil if ncbi_id.nil?
    "http://purl.bioontology.org/ontology/NCBITAXON/#{ncbi_id}"
  end

  # if present, and just a number, then converts to the http://purl.bioontology.org/ontology/NCBITAXON/[:id] form,
  # otherwise does nothing
  def convert_ncbi_id
    if concept_uri && /\A\d+\Z/.match(concept_uri)
      self.concept_uri = "http://purl.bioontology.org/ontology/NCBITAXON/#{concept_uri}"
    end
  end
end
